// Copyright 1998-2019 Epic Games, Inc. All Rights Reserved.

#include "UmgListViewDemoGameModeBase.h"
#include "Kismet/GameplayStatics.h"
#include "UI_Main.h"
#include "MyData.h"

//FilePath Test
#include "Misc/Paths.h"

//Json Test
#include "Misc/FileHelper.h"
#include "Serialization/JsonReader.h"
#include "Dom/JsonObject.h"
#include "Serialization/JsonSerializer.h"

//FJsonObjectConverter
#include "JsonObjectConverter.h"

#pragma optimize( "", off )

AUmgListViewDemoGameModeBase::AUmgListViewDemoGameModeBase()
{
	UE_LOG(LogTemp, Warning, TEXT("%s (%d)"), *FString(__FUNCTION__), __LINE__);

	static ConstructorHelpers::FClassFinder<UUserWidget> UI_WIDGET(TEXT("/Game/BluePrint/BP_UI_Main.BP_UI_Main_C"));

	if (UI_WIDGET.Succeeded())
	{
		UI_MainClass = UI_WIDGET.Class;
	}
}

FString TestFilePath()
{
	//FString MyFilePath = FPaths::ProjectConfigDir();
	FString FilePath = FPaths::ProjectDir();
	FilePath.Append(TEXT("mytest.json"));
	IPlatformFile& FileManager = FPlatformFileManager::Get().GetPlatformFile();

	if (FileManager.FileExists(*FilePath))
	{
		UE_LOG(LogTemp, Warning, TEXT("FilePaths: File found!"));
	}
	else
	{
		UE_LOG(LogTemp, Warning, TEXT("FilePaths: File not found!"));
	}

	return FilePath;
}

FString TestObjFilePath()
{
	//FString MyFilePath = FPaths::ProjectConfigDir();
	FString FilePath = FPaths::ProjectDir();
	FilePath.Append(TEXT("myobj.json"));
	IPlatformFile& FileManager = FPlatformFileManager::Get().GetPlatformFile();

	if (FileManager.FileExists(*FilePath))
	{
		UE_LOG(LogTemp, Warning, TEXT("FilePaths: File found!"));
	}
	else
	{
		UE_LOG(LogTemp, Warning, TEXT("FilePaths: File not found!"));
	}

	return FilePath;
}


FString TestObjDirtyFilePath()
{
	//FString MyFilePath = FPaths::ProjectConfigDir();
	FString FilePath = FPaths::ProjectDir();
	FilePath.Append(TEXT("myobj_dirty.json"));
	IPlatformFile& FileManager = FPlatformFileManager::Get().GetPlatformFile();

	if (FileManager.FileExists(*FilePath))
	{
		UE_LOG(LogTemp, Warning, TEXT("FilePaths: File found!"));
	}
	else
	{
		UE_LOG(LogTemp, Warning, TEXT("FilePaths: File not found!"));
	}

	return FilePath;
}

void AUmgListViewDemoGameModeBase::StartPlay()
{
	if (GetWorld() == nullptr)
		return;

	if (UI_MainClass)
	{
		UUserWidget* UserWidget = CreateWidget<UUserWidget>(GetWorld(), UI_MainClass);
		UI_Main = Cast<UUI_Main>(UserWidget);
		UI_Main->AddToViewport();
	}

	APlayerController* PlayerController = UGameplayStatics::GetPlayerController(this, 0);
	PlayerController->SetInputMode(FInputModeUIOnly());
	//PlayerController->SetShowMouseCursor(true); //4.26
	PlayerController->bShowMouseCursor = 1;  //4.24

	FString FilePath = TestFilePath();
	TestJson(FilePath);
	TestJson1(FilePath);
	TestJson2(FilePath);
	FString ObjFilePath = TestObjFilePath();
	TestObjJson(ObjFilePath);

	FString ObjDirtyFilePath = TestObjDirtyFilePath();
	TestObjJson1(ObjDirtyFilePath);
}

//⺻ б
void AUmgListViewDemoGameModeBase::TestJson(const FString& FilePath)
{
	FString filePath = FilePath;
	FString jsonstr;
	FFileHelper::LoadFileToString(jsonstr, *filePath);	

	TSharedRef<TJsonReader<TCHAR>> reader = TJsonReaderFactory<TCHAR>::Create(jsonstr);
	TSharedPtr<FJsonObject> JsonObject = MakeShareable(new FJsonObject());

	if (FJsonSerializer::Deserialize(reader, JsonObject) && JsonObject.IsValid())
	{
		int age = JsonObject->GetIntegerField(TEXT("age"));
		FString Name;
		JsonObject->TryGetStringField("name", Name);

		//convert string
		FString OutputString;
		TSharedRef< TJsonWriter<> > Writer = TJsonWriterFactory<>::Create(&OutputString);
		FJsonSerializer::Serialize(JsonObject.ToSharedRef(), Writer);

		//UE_LOG(LogTemp, Warning, TEXT("resulting jsonString -> %s"), *OutputString);
	}
}

//迭 б
void AUmgListViewDemoGameModeBase::TestJson1(const FString& FilePath)
{
	FString jsonstr;
	FFileHelper::LoadFileToString(jsonstr, *FilePath);

	TSharedRef<TJsonReader<TCHAR>> reader = TJsonReaderFactory<TCHAR>::Create(jsonstr);
	TSharedPtr<FJsonObject> JsonObject = MakeShareable(new FJsonObject());

	if (FJsonSerializer::Deserialize(reader, JsonObject) && JsonObject.IsValid())
	{
		const TArray<TSharedPtr<FJsonValue>>* list;
		bool res = JsonObject->TryGetArrayField(TEXT("arr"), list);
		for (TSharedPtr<FJsonValue> JsonItem : *list)
		{
			int num = JsonItem->AsNumber();
			//UE_LOG(LogTemp, Warning, TEXT("arr num : %d"), num);
		}
	}
}

//ü б1
void AUmgListViewDemoGameModeBase::TestJson2(const FString& FilePath)
{
	FString jsonstr;
	FFileHelper::LoadFileToString(jsonstr, *FilePath);

	TSharedRef<TJsonReader<TCHAR>> reader = TJsonReaderFactory<TCHAR>::Create(jsonstr);
	TSharedPtr<FJsonObject> JsonObject = MakeShareable(new FJsonObject());

	if (FJsonSerializer::Deserialize(reader, JsonObject) && JsonObject.IsValid())
	{
		FStudent student;
		bool res = FJsonObjectConverter::JsonObjectToUStruct(JsonObject.ToSharedRef(), &student, 0, 0);
		//UE_LOG(LogTemp, Warning, TEXT("student : %s, %d"), *student.name, student.age);
	}
}

//Object 迭 б
void AUmgListViewDemoGameModeBase::TestObjJson(const FString& FilePath)
{
	FString jsonstr;
	FFileHelper::LoadFileToString(jsonstr, *FilePath);

	TSharedRef<TJsonReader<TCHAR>> reader = TJsonReaderFactory<TCHAR>::Create(jsonstr);
	TSharedPtr<FJsonObject> JsonObject = MakeShareable(new FJsonObject());

	if (FJsonSerializer::Deserialize(reader, JsonObject) && JsonObject.IsValid())
	{
		const TArray<TSharedPtr<FJsonValue>>* list;
		bool res = JsonObject->TryGetArrayField(TEXT("student"), list);
		for (TSharedPtr<FJsonValue> JsonItem : *list)
		{
			TSharedPtr<FJsonObject> ChildObj = JsonItem->AsObject();
			FString firstName = ChildObj->GetStringField(TEXT("firstName"));
			//UE_LOG(LogTemp, Warning, TEXT("firstName : %s"), *firstName);
		}
	}
}

//ü б2
void AUmgListViewDemoGameModeBase::TestObjJson1(const FString& FilePath)
{
	FString jsonstr;
	FFileHelper::LoadFileToString(jsonstr, *FilePath);

	TSharedRef<TJsonReader<TCHAR>> reader = TJsonReaderFactory<TCHAR>::Create(jsonstr);
	TSharedPtr<FJsonObject> JsonObject = MakeShareable(new FJsonObject());

	if (FJsonSerializer::Deserialize(reader, JsonObject) && JsonObject.IsValid())
	{
		FMember member;
		bool mres = FJsonObjectConverter::JsonObjectToUStruct(JsonObject.ToSharedRef(), &member, 0, 0);

		UE_LOG(LogTemp, Warning, TEXT("resulting jsonString -> %s, %s, %d"), *member.student[0].firstName, *member.student[0].lastName, member.student[0].age);
		UE_LOG(LogTemp, Warning, TEXT("resulting jsonString -> %s, %s, %d"), *member.student[1].firstName, *member.student[1].lastName, member.student[1].age);
	}
}
#pragma optimize( "", on )